/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ISequencedDocumentMessage } from "@fluidframework/protocol-definitions";
import { ITelemetryBaseLogger } from "@fluidframework/core-interfaces";
import { IMessageProcessingResult } from "./definitions.js";
/**
 * State machine that "unrolls" contents of compressed batches of ops after decompressing them.
 * This class relies on some implicit contracts defined below:
 * 1. A compressed batch will have its first message with batch metadata set to true and compressed set to true
 * 2. Messages in the middle of a compressed batch will have neither batch metadata nor the compression property set
 * 3. The final message of a batch will have batch metadata set to false
 * 4. An individually compressed op will have undefined batch metadata and compression set to true
 */
export declare class OpDecompressor {
    private activeBatch;
    private rootMessageContents;
    private processedCount;
    private readonly logger;
    constructor(logger: ITelemetryBaseLogger);
    processMessage(message: ISequencedDocumentMessage): IMessageProcessingResult;
    private isCompressed;
}
//# sourceMappingURL=opDecompressor.d.ts.map